@@ -21,15 +21,18 @@ from .__version__ import __version__
 from .column import Column
 from .column import NoDefault
 from .exceptions import VersionError
+from .selector import AND
 from .selector import EQ
 from .selector import OR
 from .selector import Selector
 from .selector import selector_to_sql
+from .tables import CommentsColumns
 from .tables import HistoryColumns
 from .tables import JournalsColumns
 from .tables import SettingsColumns
 from .tables import SubmissionsColumns
 from .tables import UsersColumns
+from .tables import comments_table
 from .tables import history_table
 from .tables import journals_table
 from .tables import settings_table
@@ -79,6 +82,8 @@ def copy_cursors(db_dest: 'Database', cursors: Iterable['Cursor'], replace: bool
             dest_table = db_dest.submissions
         elif cursor.table.name.lower() == db_dest.journals.name.lower():
             dest_table = db_dest.journals
+        elif cursor.table.name.lower() == db_dest.comments.name.lower():
+            dest_table = db_dest.comments
         elif cursor.table.name.lower() == db_dest.settings.name.lower():
             dest_table = db_dest.settings
         elif cursor.table.name.lower() == db_dest.history.name.lower():
@@ -95,10 +100,13 @@ def copy_cursors(db_dest: 'Database', cursors: Iterable['Cursor'], replace: bool
 
 
 class Cursor:
-    def __init__(self, cursor: SQLCursor, columns: list[Column], table: 'Table'):
+    def __init__(self, cursor: SQLCursor, columns: list[Column], table: 'Table', *, query: str = None,
+                 query_values: list[Any] = None):
         self.cursor: SQLCursor = cursor
         self.columns: list[Column] = columns
         self.table: Table = table
+        self.query: str | None = query
+        self.query_values: list[Any] | None = query_values
 
     def __next__(self) -> dict[str, Value]:
         return next(self.entries)
@@ -144,7 +152,8 @@ class Table:
     def __getitem__(self, key: Value | dict[str, Value] | tuple[Value]
                     ) -> dict[str, Value] | list[dict[str, Value]] | None:
         if isinstance(key, dict):
-            return self.select({EQ: self.format_entry(key, defaults=False)}).fetchall()
+            return self.select(
+                {AND: [{EQ: {k: v}} for k, v in self.format_entry(key, defaults=False).items()]}).fetchall()
         elif isinstance(key, (tuple, list)):
             return self.select({OR: [{EQ: {self.key.name: self.key.to_entry(k)}} for k in key]}).fetchall()
         else:
@@ -164,7 +173,7 @@ class Table:
     def __iter__(self) -> Generator[dict[str, Value], None, None]:
         return self.select().entries
 
-    def _get_exists(self, key: Value):
+    def _get_exists(self, key: Value) -> dict:
         if not (entry := self[key]):
             raise KeyError(f"Entry {self.key.name} = {key!r} does not exist in {self.name} table.")
         return entry
@@ -229,27 +238,22 @@ class Table:
         sql, values = selector_to_sql(query) if query else ("", None)
         return self.select_sql(sql, values, columns, order, limit, offset)
 
-    def select_query(self, query: str, columns: list[str | Column], default_field: str, likes: list[str] = None,
-                     aliases: dict[str, str] = None, order: list[str] = None, limit: int = 0, offset: int = 0
-                     ) -> Cursor:
-
-        elements, values = query_to_sql(query, default_field, likes, aliases)
+    def select_query(self, query: str, columns: list[str | Column] = None, default_field: str = None,
+                     likes: list[str] = None, aliases: dict[str, str] = None, order: list[str] = None, limit: int = 0,
+                     offset: int = 0) -> Cursor:
+        elements, values = query_to_sql(query, default_field or self.key.name, likes, aliases)
         return self.select_sql(" ".join(elements), values, columns, order, limit, offset)
 
     def select_sql(self, sql: str, values: list[Any] = None, columns: list[str | Column] = None,
                    order: list[str] = None, limit: int = 0, offset: int = 0) -> Cursor:
         columns_: list[Column] = [(self.get_column(c) or Column(c, Any)) if isinstance(c, str) else c
                                   for c in columns] if columns else self.columns
-
-        cursor: SQLCursor = self.database.execute(
-            f"""SELECT {','.join(c.name for c in columns_)} FROM {self.name}
-                            {f' WHERE {sql}' if sql else ''}
-                            {f' ORDER BY {",".join(order)}' if order else ''}
-                            {f' LIMIT {limit}' if limit > 0 else ''}
-                            {f' OFFSET {offset}' if limit > 0 and offset > 0 else ''}""",
-            values)
-
-        return Cursor(cursor, columns_, self)
+        sql = " ".join(list(filter(bool, [f"SELECT {','.join(c.name for c in columns_)} FROM {self.name}",
+                                          f"WHERE {sql}" if sql else None,
+                                          f"ORDER BY {','.join(order)}" if order else None,
+                                          f"LIMIT {limit}" if limit > 0 else None,
+                                          f"OFFSET {offset}" if limit > 0 and offset > 0 else None])))
+        return Cursor(self.database.execute(sql, values), columns_, self, query=sql, query_values=values)
 
     def update(self, query: Selector, new_entry: dict[str, Value]) -> SQLCursor:
         sql, values = selector_to_sql(query) if query else ("", [])
@@ -283,22 +287,10 @@ class UsersTable(Table):
     def save_user(self, user: dict[str, Any], *, replace: bool = False, exist_ok: bool = False):
         self.insert(self.format_entry(user), replace=replace, exists_ok=exist_ok)
 
-    def deactivate(self, user: str) -> bool:
-        entry: dict = self._get_exists(user := clean_username(user))
-        folders: set[str] = entry[col := UsersColumns.FOLDERS.value.name]
-        if all(f.startswith("!") for f in folders):
+    def set_active(self, user: str, active: bool) -> bool:
+        if (entry := self._get_exists(user := clean_username(user)))[UsersColumns.ACTIVE.name] is active:
             return False
-        entry[col] = {f"!{f.strip('!')}" for f in folders}
-        self[user] = entry
-        return True
-
-    def activate(self, user: str) -> bool:
-        entry: dict = self._get_exists(user := clean_username(user))
-        folders: set[str] = entry[col := UsersColumns.FOLDERS.value.name]
-        if not any(f.startswith("!") for f in folders):
-            return False
-        entry[col] = {f.strip('!') for f in folders}
-        self[user] = entry
+        self[user] = entry | {UsersColumns.ACTIVE.name: active}
         return True
 
     def add_folder(self, user: str, folder: str) -> bool:
@@ -329,7 +321,7 @@ class SubmissionsTable(Table):
             self.save_submission_file(
                 submission[SubmissionsColumns.ID.value.name], file, "submission",
                 s[1] if (s := search(r"/[^/]+\.([^.]+)$", submission[SubmissionsColumns.FILEURL.value.name])) else "")
-        self.save_submission_thumbnail(submission["ID"], thumbnail)
+        self.save_submission_thumbnail(submission[SubmissionsColumns.ID.name], thumbnail)
         submission[SubmissionsColumns.FILESAVED.value.name] = (0b10 * bool(file)) + (0b01 * bool(thumbnail))
 
         self.insert(submission, replace=replace, exists_ok=exist_ok)
@@ -362,7 +354,7 @@ class SubmissionsTable(Table):
             return True
         return False
 
-    def set_user_update(self, submission_id: int, update: int) -> bool:
+    def set_user_update(self, submission_id: int, update: bool) -> bool:
         if self._get_exists(submission_id)[SubmissionsColumns.USERUPDATE.value.name] != update:
             self.update({EQ: {self.key.name: submission_id}}, {SubmissionsColumns.USERUPDATE.value.name: update})
             return True
@@ -385,7 +377,7 @@ class JournalsTable(Table):
     def save_journal(self, journal: dict[str, Any], *, replace: bool = False, exist_ok: bool = False):
         self.insert(self.format_entry(journal), replace=replace, exists_ok=exist_ok)
 
-    def set_user_update(self, journal_id: int, update: int) -> bool:
+    def set_user_update(self, journal_id: int, update: bool) -> bool:
         if self._get_exists(journal_id)[JournalsColumns.USERUPDATE.value.name] != update:
             self.update({EQ: {self.key.name: journal_id}}, {JournalsColumns.USERUPDATE.value.name: update})
             return True
@@ -398,10 +390,33 @@ class JournalsTable(Table):
         return self.remove_from_list(journal_id, JournalsColumns.MENTIONS.value, [clean_username(user)])
 
 
+class CommentsTable(Table):
+    def save_comment(self, comment: dict[str, any], *, replace: bool = False, exist_ok: bool = False):
+        self.insert(self.format_entry(comment), replace=replace, exists_ok=exist_ok)
+
+    def get_comments(self, parent_table: str, parent_id: int) -> list[dict]:
+        return list(self.select_sql(
+            f"{CommentsColumns.PARENT_TABLE.name} = ? and {CommentsColumns.PARENT_ID.name} = ?",
+            [parent_table, parent_id],
+            order=[f"{CommentsColumns.ID.name} ASC"]))
+
+    def get_comments_tree(self, parent_table: str, parent_id: int) -> list[dict]:
+        comments: list[dict] = self.get_comments(parent_table, parent_id)
+        return self._make_comments_tree([c for c in comments if c[CommentsColumns.REPLY_TO.name] is None], comments)
+
+    def make_comments_tree(self, comments: list[dict]) -> list[dict]:
+        return self._make_comments_tree(comments, comments)
+
+    def _make_comments_tree(self, comments: list[dict], all_comments: list[dict]) -> list[dict]:
+        return [com | {"REPLIES": self._make_comments_tree(
+            [c for c in all_comments if c[CommentsColumns.REPLY_TO.name] == com[CommentsColumns.ID.name]],
+            all_comments)} for com in comments]
+
+
 class SettingsTable(Table):
-    version_setting: str = "VERSION"
-    files_folder_setting: str = "FILESFOLDER"
-    default_files_folder: str = "FA.files"
+    _version_setting: str = "VERSION"
+    _files_folder_setting: str = "FILESFOLDER"
+    _default_files_folder: str = "FA.files"
 
     def __getitem__(self, item: str) -> str | None:
         return (super().__getitem__(item) or {}).get(SettingsColumns.SVALUE.value.name, None)
@@ -411,22 +426,22 @@ class SettingsTable(Table):
 
     @property
     def version(self):
-        return self[self.version_setting]
+        return self[self._version_setting]
 
     @property
     def files_folder(self) -> Path:
-        folder: Path = Path(self[self.files_folder_setting])
+        folder: Path = Path(self[self._files_folder_setting])
         return folder if folder.is_absolute() else (self.database.path.parent / folder).resolve()
 
     @files_folder.setter
     def files_folder(self, value: str | Path):
-        self[self.files_folder_setting] = str(value)
+        self[self._files_folder_setting] = str(value)
 
     def create(self, exists_ignore: bool = False):
         super().create(exists_ignore=exists_ignore)
-        self.insert({SettingsColumns.SETTING.value.name: self.files_folder_setting,
-                     SettingsColumns.SVALUE.value.name: self.default_files_folder}, exists_ok=True)
-        self.insert({SettingsColumns.SETTING.value.name: self.version_setting,
+        self.insert({SettingsColumns.SETTING.value.name: self._files_folder_setting,
+                     SettingsColumns.SVALUE.value.name: self._default_files_folder}, exists_ok=True)
+        self.insert({SettingsColumns.SETTING.value.name: self._version_setting,
                      SettingsColumns.SVALUE.value.name: __version__}, exists_ok=True)
 
 
@@ -453,6 +468,7 @@ class Database:
         self.users: UsersTable = UsersTable(self, users_table, UsersColumns.as_list())
         self.submissions: SubmissionsTable = SubmissionsTable(self, submissions_table, SubmissionsColumns.as_list())
         self.journals: JournalsTable = JournalsTable(self, journals_table, JournalsColumns.as_list())
+        self.comments: CommentsTable = CommentsTable(self, comments_table, CommentsColumns.as_list())
         self.settings: SettingsTable = SettingsTable(self, settings_table, SettingsColumns.as_list())
         self.history: HistoryTable = HistoryTable(self, history_table, HistoryColumns.as_list())
 
@@ -507,7 +523,7 @@ class Database:
 
     @property
     def is_formatted(self):
-        return settings_table in self
+        return settings_table in self and self.version is not None
 
     @property
     def is_open(self) -> bool:
@@ -525,6 +541,7 @@ class Database:
         self.users.create(exists_ignore=True)
         self.submissions.create(exists_ignore=True)
         self.journals.create(exists_ignore=True)
+        self.comments.create(exists_ignore=True)
         self.settings.create(exists_ignore=True)
         self.history.create(exists_ignore=True)
 
