@@ -9,11 +9,13 @@ __all__ = [
     "users_table",
     "submissions_table",
     "journals_table",
+    "comments_table",
     "settings_table",
     "history_table",
     "UsersColumns",
     "SubmissionsColumns",
     "JournalsColumns",
+    "CommentsColumns",
     "SettingsColumns",
     "HistoryColumns",
 ]
@@ -21,6 +23,7 @@ __all__ = [
 users_table: str = "USERS"
 submissions_table: str = "SUBMISSIONS"
 journals_table: str = "JOURNALS"
+comments_table: str = "COMMENTS"
 settings_table: str = "SETTINGS"
 history_table: str = "HISTORY"
 
@@ -34,6 +37,7 @@ class ColumnsEnum(Enum):
 class UsersColumns(ColumnsEnum):
     USERNAME = Column("USERNAME", str, unique=True, key=True, check="length({name}) > 0", to_entry=clean_username)
     FOLDERS = Column("FOLDERS", set)
+    ACTIVE = Column("ACTIVE", bool)
     USERPAGE = Column("USERPAGE", str, to_entry=str.strip)
 
 
@@ -55,17 +59,28 @@ class SubmissionsColumns(ColumnsEnum):
     FAVORITE = Column("FAVORITE", set)
     MENTIONS = Column("MENTIONS", set)
     FOLDER = Column("FOLDER", str, to_entry=str.lower, check="{name} in ('gallery', 'scraps')")
-    USERUPDATE = Column("USERUPDATE", int, check="{name} in (0, 1)")
+    USERUPDATE = Column("USERUPDATE", bool)
 
 
 class JournalsColumns(ColumnsEnum):
-    ID = Column("ID", int, unique=True, key=True, check="ID > 0")
+    ID = Column("ID", int, unique=True, key=True, check="{name} > 0")
     AUTHOR = Column("AUTHOR", str, check="length({name}) > 0")
     TITLE = Column("TITLE", str)
     DATE = Column("DATE", datetime)
     CONTENT = Column("CONTENT", str)
     MENTIONS = Column("MENTIONS", set)
-    USERUPDATE = Column("USERUPDATE", int, check="{name} in (0, 1)")
+    USERUPDATE = Column("USERUPDATE", bool)
+
+
+class CommentsColumns(ColumnsEnum):
+    ID = Column("ID", int, key=True, check="{name} > 0")
+    PARENT_TABLE = Column("PARENT_TABLE", str, key=True, check=f"{'{name}'} in ('{submissions_table}', '{journals_table}')")
+    PARENT_ID = Column("PARENT_ID", int, key=True, check="{name} > 0")
+    REPLY_TO = Column("REPLY_TO", int, not_null=False, check="{name} == null or {name} > 0")
+    AUTHOR = Column("AUTHOR", str, check="length({name}) > 0")
+    DATE = Column("DATE", datetime, to_entry=lambda v: v.strftime("%Y-%m-%dT%H:%M:%S"),
+                  from_entry=lambda v: datetime.strptime(v, "%Y-%m-%dT%H:%M:%S"))
+    TEXT = Column("TEXT", str)
 
 
 class SettingsColumns(ColumnsEnum):
